// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnlibs.hpp"

namespace Dawn {
  struct Color {
    uint8_t r, g, b, a;
    
    glm::vec4 precision() {
      return {
        (float_t)r / 255.0f,
        (float_t)g / 255.0f,
        (float_t)b / 255.0f,
        (float_t)a / 100.0f
      };
    }

    struct Color operator * (const float_t &x) {
      return {
        (uint8_t)(r * x),
        (uint8_t)(g * x),
        (uint8_t)(b * x),
        (uint8_t)(a * x)
      };
    }

    struct Color operator - (const struct Color &color) {
      return {
        (uint8_t)(r - color.r),
        (uint8_t)(g - color.g),
        (uint8_t)(b - color.b),
        (uint8_t)(a - color.a)
      };
    }

    struct Color operator + (const struct Color &color) {
      return {
        (uint8_t)(r + color.r),
        (uint8_t)(g + color.g),
        (uint8_t)(b + color.b),
        (uint8_t)(a + color.a)
      };
    }
  };

  #define COLOR_WHITE { 255, 255, 255, 100 }
  #define COLOR_RED { 255, 0, 0, 100 }
  #define COLOR_GREEN { 0, 255, 0, 100 }
  #define COLOR_BLUE { 0, 0, 255, 100 }
  #define COLOR_BLACK { 0, 0, 0, 100 }
  #define COLOR_MAGENTA { 255, 0, 255, 100 }
  #define COLOR_DARK_GREY { 50, 50, 50, 100 }
  #define COLOR_LIGHT_GREY { 204, 204, 204, 100 }
  #define COLOR_CORNFLOWER_BLUE { 100, 149, 237, 100 }
  #define COLOR_WHITE_TRANSPARENT { 255, 255, 255, 0 }
  #define COLOR_BLACK_TRANSPARENT { 0, 0, 0, 0 }
  #define COLOR_TRANSPARENT COLOR_BLACK_TRANSPARENT
}